%cipherkey
% Generate our cipher key and write out this in a file. This will be a
% set of N random alphabets to enable a polyalphabetic cipher to be
% encrypted. Note the first two characters of the output filename define
% the number of cipher alphabets.
%
% LAST UPDATED by Andy French April 2016

function cipherkey

%Display message to command window
disp(' ' )
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Polyalphabetic cipher key generator ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Ask user to define the number of alphabets
disp(' Define number of random permutations of plaintext alphabet. (Max 99).' )
disp( '  ')
N = input( ' > N = ');
if isempty(N)
    N = 10;
    disp(' Default N = 10.')
end
disp(' ')

%Restricted or full ASCII alphabet?
disp( ' Choose alphabet used for encipherment .... ')
disp( ' ' )
disp(' Option 1: AaBbCc ... zZ ')
disp(' Option 2: AaBb....Zz!.,"?#@>..... ')
disp(' Option 3: AaBb....Zz!.,"?#@>..... + line return characters')
disp(' Option 4: All 255 ASCII characters char(0)...char(255)' )
disp(' ')
option = input(' > ');
if isempty(option)
    option = 2;
    disp(' Default is Option 2: AaBb....Zz!.,"?#@>.....')
    disp(' ')
end

%Define the original alphabet
if option ==1
    alphabet = 'AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz';
elseif option == 2
    alphabet = ['AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz',...
    '!.,"?#@><;:()$%^&*_-{}[]~1234567890\/|+= '];
elseif option == 3
    alphabet = ['AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz',...
    '!.,"?#@><;:()$%^&*_-{}[]~1234567890\/|+= ',...
    char(10), char(13) ];
elseif option == 4
   alphabet = '';
    for n=1:255
        alphabet = [alphabet,char(n)];
    end       
else
    alphabet = ['AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz',...
    '!.,"?#@><;:()$%^&*_-{}[]~1234567890\/|+= '];
end

%Work out N as a two character string
N = round(N);
if N<1
    N_str = '01';
elseif N<10
    N_str = ['0',num2str(N)];
elseif N<=99
    N_str = num2str(N);
else
    %Max N is 99
    N_str = '01';
end

%Define a character array A. This is the number of cipher alphabets, the
%original alphabet and then the N cipher alphabets, all as a single string.
A = [N_str,alphabet];

for n=1:N
    %Shuffle alphabet randomly
    shuffled_alphabet = alphabet;
    shuffled_alphabet = shuffled_alphabet( randperm(numel(alphabet)) );
    %Add to character arra A
    A = [A,shuffled_alphabet];
end

%Open a text file and write the original alphabet and then the shuffled
%alphabet. Define the cipher key filename to be based on the current date and time
fid = fopen( ['cipherkey ',strrep( datestr(now),':','-' ),'.txt'], 'w+' );
fwrite( fid, A );

%Close the file
fclose(fid);

%Display completion message
disp(' Cipher key generated.' )
disp(' ')

%End of code