%polycipher
% Function which loads a .txt file and applies a polyalphabetic cipher, defined via a
% cipher key file of the sort generated by cipherkey.m or vigenere.m.
%
% LAST UPDATED by Andy French. April 2016

function polycipher

%Display opening message to command window
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%' )
disp('  Polyalphabetic cipher encryption and decryption ' )
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%' )
disp(' ')
disp(' Choose file(s) to encrypt or decrypt...' )
disp(' ')

%Find plaintext text file to encrypt or decrypt
[filename, pathname] = uigetfile( '*.txt',...
    'Select location of text file(s)...', ...
    'multiselect', 'on');
if iscell(filename)==0
    %Store filename in a cell array, if just one file has been chosen.
    %This means we don't have to consider the single file case separately
    %later.
    if filename == 0
        return
    else
        filename = {filename};
    end
end

%Choose and load file containing cipher key
[keyfilename, keypathname] = uigetfile( '*.txt',...
    'Choose cipher key file...', ...
    'multiselect', 'off');
fid = fopen( [keypathname,keyfilename] );
key = fscanf(fid, '%c');
fclose(fid);

%Extract number of alphabets and hence plaintext and cipherkey alphabets.
%Store the latter in a cell array cipherbet
N = str2num(key(1:2));
L = (numel(key)-2)/(N+1);
start = 3;
stop = 2+L;
alphabet = key(start:stop);
cipherbet = cell(1,N);
for n=1:N
    start = stop + 1;
    stop = start + L - 1;
    cipherbet{n} = '';
    cipherbet{n} = key(start:stop);
end

%Choose encrypt or decrypt option
choice = questdlg('choose encryption option', ...
    'encryption option', ...
    'encrypt','decrypt',...
    'encrypt');
if strcmp(choice,'')
    return
end

%Start clock
t0 = tic;

%Loop through selected files and encrypt or decrypt
for n=1:numel(filename)
    
    %Open file filename (read only)
    fid = fopen( [pathname,filename{n}], 'r' );
    
    %Store filename text in a row vector of ascii characters
    t = fscanf(fid, '%c');
    
    %Close file
    fclose(fid); clear fid
    
    %Choose between encryption or decryption modes
    switch choice
        case 'encrypt'
            method = '-e';
            
            %Step through t and exchange letters that belong to the alphabet with
            %corresponing ones from the cipherbet. Cycle through the N cipher
            %alphabets
            k = 1;
            for m=1:length(t)
                
                %Find index of plaintext character in alphabet vector
                i = strfind( alphabet, t(m) );
                
                %Replace by corresponding character in cipherbet{k}
                if isempty(i)==0
                    t(m) = cipherbet{k}(i);
                end
                
                %Move to the next cipherbet
                k = k + 1;
                if k > N
                    k = 1;
                end
            end                     
        case 'decrypt'
            method = '-d';
            
            %Step through t and exchange letters that belong to the alphabet with
            %corresponing ones from the cipherbet. Cycle through the N cipher
            %alphabets
            k = 1;
            for m=1:length(t)
                
                %Find index of plaintext character in cipherbet{k}
                i = strfind( cipherbet{k}, t(m) );
                
                %Replace by corresponding character in alphabet
                if isempty(i)==0
                    t(m) = alphabet(i);
                end
                             
                %Move to the next cipherbet
                k = k + 1;
                if k > N
                    k = 1;
                end
            end           
    end
    
    %Write out encrypted (or decrypted) file
    fid = fopen( strrep( [pathname,filename{n} ],'.txt',...
        [method,'.txt'] ),'w+' );
    fwrite( fid, t );
    fclose(fid);
    
    %Display message to command window
    if strcmp( method, '-e')==1
        disp([' File ',filename{n},' encrypted.'])
    else
        disp([' File ',filename{n},' decrypted.'])
    end
    
end

%Stop clock and display completion message
disp(' ');
disp([' All ',num2str(numel(filename)),...
    ' files processed in ',num2str(toc(t0)),' seconds.']);
disp(' ');

%End of code