%textstats
% Prompt user for a .txt file. Loads text file and generates a frequency
% table of the letters within the file. This could be compared to a
% standard English source (e.g. Works of Shakspeare, Bible etc) for use in
% frequency analysis, which could help to break a substitution cipher.
%
% LAST UPDATED by Andy French April 2016

function textstats

%Hard code the alphabet for which we will use for our frequency analysis
alphabet = ['AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz',...
    '!.,"?#@><;:()$%^&*_-{}[]~1234567890\/|+= ',char(10), char(13),];

%Fontsize for graphs
fsize = 8;

%

%Create a cell array with each element being a letter from alphabet
alphabet_cell = cell(size(alphabet));
for n = 1:length(alphabet);
    alphabet_cell{n} = alphabet(n);
end

%Bring up a windows file browser
[filename, pathname] = uigetfile('*.txt', 'Choose text file to analyse');
if filename ==0
    return
end

%Load the text file
fid = fopen( [pathname,filename], 'r' );

%Store text in a row vector t of characters
t = fscanf(fid, '%c');

%Close the file
fclose(fid);

%Initialize a frequency table
f=zeros(1,length(alphabet));

%Step through elements of alphabets and populate in f the number of
%instances of occurances of alphabet characters in file 'filename'
for n=1: length(alphabet)
    f(n)=length(strfind(t,alphabet(n)));
end

%Create and plot a bar chart
figure('name','text stats', 'color', [1,1,1]);
b = bar(1:length(alphabet),f);
set( b, 'facecolor','g');
title(['textstats for file: ', filename],'fontsize',fsize);
set(gca, 'xtick',1:length(alphabet),'xticklabel', alphabet_cell,...
    'fontsize' , fsize);
axis tight
grid on
set(gca,'fontsize',fsize)
print( gcf, [pathname,'textstats ', strrep(filename,'.txt','.png')],'-dpng','-r300' );

%End of code