%vigenerekey
% Generates a cipher key based upon a Vigenere square. Only standard
% alphabet letters are encrypted, and upper case letters ae mapped to the
% same cipher alphabet characters as lower case.
%
% LAST UPDATED by Andy French April 2016

function vigenerekey

%Display message to command window
disp(' ' )
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Vigenere cipher key generator ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Ask user for the Vigenere code word (spaces are ignored, case is ignored)
disp(' Set the Vigenere code word.')
disp(' Note spaces and non-alphabetic characters are ignored. ')
disp( '  ')
codeword = upper( input( ' > ','s' ) );
if isempty(codeword)
    codeword = 'VIGENERE';
    disp(' Default codeword is VIGENERE')
end
disp(' ')

%Define basic alphabet
alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';

%Determine alphabet positions p corresponding to the valid characters of the codeword
%We ignore any characters which are not A...Z and we don't care about case.
p = [];
for n=1:length(codeword)
    i = strfind( alphabet, codeword(n) );
    if isempty(i)==0
        p = [p,i];
    end
end

%Work out number of cipher alphabets N as a two character string
N = numel(p);
if N==0
    disp(' Codeword does not contain any normal alphabetic characters. Try again!')
    vigenere;
elseif N<10
    N_str = ['0',num2str(N)];
elseif N<=99
    N_str = num2str(N);
else
    disp(' Codeword is too long. Max 99 characters. Try again!')
    vigenere;
end

%Define a character array A. This is the number of cipher alphabets, the
%original alphabet and then the N cipher alphabets, all as a single string.
A = [N_str,alphabet,lower(alphabet)];

%Add the Caesar shifted alphabets to A. Note the elements of p define the
%shift.
for n=1:N
    cipherbet = circshift( alphabet.', -p(n)+1 ).';
    A = [A,cipherbet,lower(cipherbet)];
end

%Open a text file and write the original alphabet and then the shuffled
%alphabet.
fid = fopen( ['vigenere cipherkey ',codeword,'.txt'], 'w+' );
fwrite( fid, A );

%Close the file
fclose(fid);

%Display completion message
disp(' Cipher key generated.' )
disp(' ')

%End of code