%wordcounter
% Prompt user for a .txt file. Loads text file and generates a frequency
% table of the words within the file. This is written to an Excel sheet to
% enable further analysis.
%
% LAST UPDATED by Andy French April 2016

function wordcounter

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%')
disp('  Wordcounter ')
disp(' %%%%%%%%%%%%%')
disp(' ')

%Bring up a windows file browser to select the text file to analyse
[filename, pathname] = uigetfile('*.txt', 'Choose text file to analyse');
if filename ==0
    return
end

%Display message to command window, and start clock
t0 = tic;
disp([' Loading and counting words in file: ',filename ] )
disp(' ')

%Load the text file
fid = fopen( [pathname,filename], 'r' );

%Store text in a row vector t of lower case characters
t = lower( fscanf(fid, '%c') );

%Close the file
fclose(fid);

%Remove all the line returns, replacing them with spaces
t( strfind( t, char(10) ) ) = ' ';
t( strfind( t, char(13) ) ) = ' ';

%Do the same for --
t( strfind( t, '--' ) ) = ' ';

%Remove punctuation marks, to prevent them creating distinct words
t( strfind( t, '.' ) ) = [];
t( strfind( t, ',' ) ) = [];
t( strfind( t, ';' ) ) = [];
t( strfind( t, ':' ) ) = [];
t( strfind( t, '!' ) ) = [];
t( strfind( t, '"' ) ) = [];
t( strfind( t, '?' ) ) = [];
t( strfind( t, '(' ) ) = [];
t( strfind( t, ')' ) ) = [];
t( strfind( t, '{' ) ) = [];
t( strfind( t, '}' ) ) = [];
t( strfind( t, '[' ) ) = [];
t( strfind( t, ']' ) ) = [];

%Search for all the spaces
spaces = strfind( t, ' ');

%Estimate the number of words to check through (some could be blanks or
%spaces, so a better calculation is done at the end)
num_words = length(spaces);
disp([' Approximately ',num2str(num_words),' need to be checked...']);
disp(' ')

%Initialize list of words, and corresponding frequency of occurance
words = {};
f = [];

%Step through t, and add to words and f
start = 1;
stop = spaces(1);
for n=1:length(spaces)
    
    %Check if n has gone past a 1000 words
    if rem( n, 1000)==0
        disp([' Word number ',num2str(n),' of ',num2str(num_words),' checked...'])
    end
    
    %Get next word
    w = t( start : stop );
    
    %Check if w is already in the list of words in cell array words
    c = strcmp( w, words );
    if sum(c)==0
        %A new word!
        N = numel(words) + 1;
        words{N} = w;
        f(N) = 1;
    else
        %Word already is in words. Add to frequency vector f
        N = find( c == 1 );
        f(N) = f(N) + 1;
    end
    
    %Update start and stop
    if n < (length(spaces)-1)
        start = spaces(n+1)+1;
        stop = spaces(n+2)-1;
    elseif n==( length(spaces)-1)
        start = spaces(n+1)+1;
        stop = length(t);
    end
end
disp([' All ',num2str(num_words),' checked. Writing results into Excel file...'])
disp(' ');

%Remove any spaces that have found their way into words
i=[];
for n=1:length(words)
    word = words{n};
    if isempty( word )
        i = [i,n];
    elseif strcmp( word(1),' ' )==1
        i = [i,n];
    end
end
words(i) = [];
f(i) = [];

%Construct cell array with words and frequencies
E{1,1} = [' Wordcount for file: ',filename,'. Total word count = ',num2str(sum(f)),'.' ];
E{2,1} = ' ';
E{3,1} = 'Word';
E{3,2} = 'Frequency';
for n=1:length(words)
    E{3+n,1} = words{n};
    E{3+n,2} = f(n);
end
xlswrite([pathname,'wordcount ',strrep(filename,'.txt','.xls')],E );

%Display ending message
disp([' File: ',filename,' analysed in ',num2str(toc(t0)), ' seconds.' ] )
disp(' ')

%End of code